# Aura Engine

<img src="https://gitlab.servus.at/aura/meta/-/raw/main/assets/images/aura-engine.png" width="250" align="right" />

Aura Engine is a scheduling and play-out engine as part of [Aura Radio Software Suite](#About), specifically built for
the requirements of community radio stations.

<!-- TOC -->

1. [Aura Engine](#aura-engine)
   1. [Functionality](#functionality)
      1. [Scheduler](#scheduler)
         1. [Versatile playlists](#versatile-playlists)
         2. [Default playlists](#default-playlists)
      2. [Heartbeat Monitoring](#heartbeat-monitoring)
      3. [Logging](#logging)
   2. [Getting started](#getting-started)
      1. [Run natively](#run-natively)
      2. [Run with Docker](#run-with-docker)
   3. [Read more](#read-more)
   4. [About](#about)

<!-- /TOC -->

## Functionality

In conjunction with other AURA components Engine provides several features:

- **Scheduler** to automatically broadcast your radio programme (see [AURA Dashboard](https://gitlab.servus.at/aura/dashboard) for a user interface to do scheduling)
- **Analog input and outputs** provided by [Engine Core](https://gitlab.servus.at/aura/engine-core)
- **Streaming to an [Icecast](https://icecast.org/) Server including [Icy Metadata](https://cast.readme.io/docs/icy)** provided by [Engine Core](https://gitlab.servus.at/aura/engine-core)
- **Autonomous playout** by caching the schedule information pulled from [Steering](https://gitlab.servus.at/aura/steering) in a local database. This allows Engine be keep running, independently from any network or service outages. This enables the application of (_High Availability_ infrastructure scenarios)[https://gitlab.servus.at/aura/meta/-/blob/master/docs/administration/installation-guide.md#high-availability].
- **Auto DJ with Silcence Detector** provided by [Engine Core](https://gitlab.servus.at/aura/engine-core)
- **Web Application for a Track Service** (see [AURA Player](https://gitlab.servus.at/aura/player))
- **Web Application providing a Studio Clock** (see [Dashboard Clock](https://gitlab.servus.at/aura/dashboard-clock))
- **Bulk and Timeslot Recorder** (_Not yet available_, this will be provided by a planned `engine-recorder` component)
- **API** to query Track-Service, monthly reports and information for displaying the Studio Clock (see [Engine API](https://gitlab.servus.at/aura/engine-api))

### Scheduler

Engine provide a scheduling functionality by polling external API endpoints frequently. Those API endpoints are provided by [Steering](https://gitlab.servus.at/aura/steering) to retrieve schedule information and [Tank](https://gitlab.servus.at/aura/tank) to retrieve playlist information. To define your schedule you'll also need [AURA Dashboard](https://gitlab.servus.at/aura/dashboard) which is an elegent web user interface to manage your shows, playlists and schedules.

Ideally any audio is scheduled some time before the actual, planned playout to avoid timing issues with buffering and preloading. Nonetheless, playlists can also be scheduled after a given calendar timeslot has started already. In such case the playout starts as soon it is preloaded.

If for some reason the playout is corrupted, stopped or too silent to make any sense, then this <u>triggers a fallback using the silence detector</u> (see chapter below).

> Note: If you delete any existing timeslot in Dashboard/Steering this is only reflected in Engine until the start of the scheduling window. The scheduling window is defined by the start of the timeslot minus a configured offset in seconds (compare your Engine configuration).

#### Versatile playlists

It is possible to schedules playlists with music or pre-recorded shows stored on the **file system**, via external **streams** or live from an **analog input** in the studio. All types of sources can be mixed in a single playlist.

The switching between types of audio source is handled automatically, with configured fadings applied.

> Note: Any live sources or streams not specifying a length property, are automatically expanded to the left duration of the timeslot.

#### Default playlists

While a timeslot can have a specific playlist assigned, it is also possible to define default playlists
for schedules and shows:

- **Default Schedule Playlist**: This playlist is defined on the level of some recurrence rules (_Schedule_).
  In case the timeslot doesn't have any specific playlist assigned, this playlist is broadcasted.

- **Default Show Playlist**: This playlist can be assigned to some show. If neither the specific timeslot
  playlist nor the default schedule playlist is specified the _default show playlist_ is broadcasted.

If none of these playlists have been specified the _Auto DJ_ feature of [Engine Core](https://gitlab.servus.at/aura/engine-core) takes over (optional).

### Heartbeat Monitoring

Instead of checking all status properties, the Heartbeat only validates the vital ones required to run the engine. If all of those are valid, a network socket request is sent to a defined server. This heartbeat is sent continuously based on the configured `heartbeat_frequency`. The service receiving this heartbeat ticks can decide what to do with that information. One scenario could be switching to another Engine instance or any other custom failover scenario. Under `contrib/heartbeat-monitor` you'll find some sample application digesting these heartbeat signals.

### Logging

All Engine logs can be found in the local `./logs` directory. Adapt the log-level within your configuration to get more or less verbose log output. Whenever the Engine's status turns into some unhealthy state, additionally this is logged to [Engine API](https://gitlab.servus.at/aura/engine-api). Also, when it returns to some valid state this is logged to the Engine API.

## Getting started

Ensure that you have also dependencies such as `steering`, `tank`, `dashboard`, `engine-core`, and `engine-api` up and running.

### Run natively

If you want to install for AURA development or simply prefer to run natively, check out the [Bare Metal Installation](docs/bare-metal-installation.md).

### Run with Docker

For production deployments follow the [Docker](https://docs.aura.radio/en/latest/administration/install-docker.html) and [Docker Compose](https://docs.aura.radio/en/latest/administration/install-docker-compose.html) Installation Documentation at [docs.aura.radio](https://docs.aura.radio/).

## Read more

- [Bare Metal Installation](docs/bare-metal-installation.md)
- [Developer Guide](docs/developer-guide.md)
- [Setting up the Audio Store [meta]](https://gitlab.servus.at/aura/meta/-/blob/master/docs/administration/setup-audio-store.md)

## About

[<img src="https://aura.radio/assets/aura-logo-grayscale.png" width="150" />](https://aura.radio)
