#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-now() - The Aura Engine Team.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import unittest

import validators

from aura_engine.base.config import AuraConfig


class TestConfig(unittest.TestCase):
    """
    Testing the Configuration.
    """

    config = None

    def setUp(self):
        self.config = AuraConfig()

    def test_config(self):
        # Check if config is available
        self.assertIsNotNone(self.config.ini_path)

        # Check if "install_dir" is a valid directory (is evaluated at runtime)
        self.assertTrue(os.path.isdir(self.config.get("install_dir")))

        # Check API Urls
        self.assertTrue(validators.url(self.config.get("api_steering_status")))
        self.assertTrue(validators.url(self.config.get("api_steering_calendar")))
        self.assertTrue(validators.url(self.config.get("api_tank_status")))
        tank_playlist_url = self.config.get("api_tank_playlist").replace("${ID}", "1")
        self.assertTrue(validators.url(tank_playlist_url))
        self.assertTrue(validators.url(self.config.get("api_engine_status")))
        self.assertTrue(validators.url(self.config.get("api_engine_store_playlog")))
        self.assertTrue(validators.url(self.config.get("api_engine_store_clock")))
        engine_health_url = self.config.get("api_engine_store_health").replace(
            "${ENGINE_NUMBER}", "1"
        )
        self.assertTrue(validators.url(engine_health_url))

        # Check if database settings are set
        self.assertIsNotNone(self.config.get("db_user"))
        self.assertIsNotNone(self.config.get("db_pass"))
        self.assertIsNotNone(self.config.get("db_name"))
        self.assertIsNotNone(self.config.get("db_host"))


if __name__ == "__main__":
    unittest.main()
