#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-now() - The Aura Engine Team.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import unittest

from aura_engine.base.api import SimpleApi
from aura_engine.base.config import AuraConfig


class TestApi(unittest.TestCase):
    """
    Testing the Configuration.
    """

    config = None
    api = None

    def setUp(self):
        self.config = AuraConfig()
        self.api = SimpleApi()

    def test_config(self):
        # Check if config is available
        self.assertIsNotNone(self.config.ini_path)

    def test_build_json(self):
        import re

        data = {"foo": "bar", "fooBar2000": None}
        json_string = '{"foo":"bar"}'

        # Success
        result = self.api.build_json(data)
        result = re.sub(r'[^a-z0-9\{\}\:"]+', "", result, flags=re.IGNORECASE)
        self.assertEqual(json_string, result)

        # Check for cleaned up key
        try:
            print(result.index("fooBar2000"))
            self.assertFalse("Error: Value found in dict")
        except ValueError:
            self.assertTrue("Value not found in dict")

    def test_get(self):
        result = self.api.get("https://o94.at/")

        # Success
        # print(result.response.status_code)
        self.assertEqual(200, result.response.status_code)

    def test_post(self):
        data = {"foo": "bar"}

        # Not allowed
        result = self.api.post("https://o94.at/", data=data)
        print(result)
        self.assertEqual(405, result.response.status_code)

    def test_put(self):
        data = {"foo": "bar"}

        # Bad request: Invalid URL
        result = self.api.put("http:/0.0.0.0/invalid-service", data=data)
        print(result)
        self.assertEqual(400, result.response.status_code)

        # Not found
        result = self.api.put("https://o94.at/invalid-service", data=data)
        print(result)
        self.assertEqual(404, result.response.status_code)


if __name__ == "__main__":
    unittest.main()
